%FT of disc of radius r and square of width w, with diameter p = w = 2r
n = 1024;
Q = 9;                                              %padding ratio
xAxis =(-0.5:1/n:0.5-1/n) *2*Q;                     %disc radii (w = 2r)
mtfAxis = (0:n-1) / Q;                              %cycles/diameter, or
                                                    %c/p (p = w = 2r)
%disc
[X,Y] = meshgrid(xAxis);                            %2D cartesian coords
RHO = sqrt(X.^2 + Y.^2);                            %2D polar coordinates
PSF = ones(n);
PSF(RHO > 1) = 0;                                   %2D disc PSF

MTF_psf = abs(fft2(PSF));                           %2D MTF
MTF_psf = MTF_psf / max(max(MTF_psf));

LSF = sum(PSF);                                     %1D LSF, projected
MTF_lsf = abs(fft(LSF));                            %1D MTF, directional
MTF_lsf = MTF_lsf/MTF_lsf(1);
 
f = (0:1/n:1-1/n) * 5;                              %analytical MTF of disc
MTF_disc = 2*abs(besselj(1,pi*f)) ./ (pi*f);
MTF_disc(1) = 1;

%square
psf = zeros(n);
psf(n/2+1-floor(n/Q/2):n/2+1+ceil(n/Q/2),n/2+1-floor(n/Q/2):n/2+1+ceil(n/Q/2)) = 1;
MTF = abs(fft2(psf));                               %2D MTF
MTF = MTF/max(MTF(:));

lsf = sum(psf);
mtf = abs(fft(lsf));                                %1D MTF
mtf = mtf/mtf(1);

MTF_sq = abs(sinc(f));                              %1D MTF, analytical

%% plots
figure
plot(xAxis, LSF); hold on
plot(xAxis, lsf)
xlabel('radii (w =2r)'); grid
title('LSF of Disc and Square')

figure
plot(f,MTF_disc); hold on
plot(mtfAxis,MTF_psf(1,:),'.');
plot(mtfAxis,MTF_lsf,'o')

plot(f,MTF_sq)
plot(mtfAxis,MTF(1,:),'.')
plot(mtfAxis,mtf,'o')

xlim([0 5]); grid; 
xlabel('c/p (cycles per diameter: p = w = 2r)'); ylabel('MTF')
title('FT of Disc and Square with p = w = 2r')
legend('disc MTF, Analytical','disc MTF, 2D','disc MTF, 1D from LSF',...
    'Square MTF, Analytical','Square MTF, 2D', 'Square MTF, 1D from LSF')
